package com.alterdekim.game.repository;

import com.alterdekim.game.component.game.AvatarInventoryType;
import com.alterdekim.game.entity.AvatarInventory;
import jakarta.transaction.Transactional;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Optional;

@Repository
public interface AvatarInventoryRepository extends JpaRepository<AvatarInventory, Long> {

    @Query(value = "SELECT a FROM AvatarInventory a WHERE a.user.id = :uid AND a.type = :type")
    List<AvatarInventory> findItemsByUserIdAndType(@Param("uid") Long userId, @Param("type") AvatarInventoryType type);

    @Query(value = "SELECT a FROM AvatarInventory a WHERE a.user.id = :uid AND a.type = :type AND a.isUsed = true")
    List<AvatarInventory> findUsedItemsByUserIdAndType(@Param("uid") Long userId, @Param("type") AvatarInventoryType type);

    @Query(value = "SELECT a.isUsed FROM AvatarInventory a WHERE a.user.id = :userId AND a.type = :type AND a.goodId = :goodId")
    Optional<Boolean> getIsUsed(@Param("userId") Long userId, @Param("type") AvatarInventoryType type, @Param("goodId") Long goodId);

    @Transactional
    @Modifying
    @Query(value = "UPDATE AvatarInventory a SET a.isUsed = false WHERE a.user.id = :userId AND a.type = :type")
    void resetUsed(@Param("userId") Long userId, @Param("type") AvatarInventoryType type);

    @Transactional
    @Modifying
    @Query(value = "UPDATE AvatarInventory a SET a.isUsed = true WHERE a.user.id = :userId AND a.type = :type AND a.goodId = :goodId")
    void setUsed(@Param("userId") Long userId, @Param("goodId") Long goodId, @Param("type") AvatarInventoryType type);

    @Query(value = "SELECT a FROM AvatarInventory a WHERE a.user.id = :uid AND a.type = :type AND a.goodId = :goodId")
    Optional<AvatarInventory> getSingleBy(@Param("type") AvatarInventoryType type, @Param("goodId") Long goodId, @Param("uid") Long userId);
}
